function [p,t,be,bn]=kmg2dref(p0,t0,dg,fd)
%KMG2DREF Refine a two-dimensional triangular mesh
%--------------------------------------------------------------------
% [p,t,be,bn]=kmg2dref(p0,t0,dg,fd)
% Input:
% p0 : Initial node coordinates np0*2
% t0 : Initial triangle vertices nt0*3
% dg : Type of the triangular mesh: dg=1 linear; dg=2 quadratic
% fd : Signed distance function d(x,y)
% Output:
% p : Nodes coordinates of the refined mesh, np*2
% t : Triangle nodes of the refined mes, nt*3 or nt*6
% be : Boundary edgdes in the refined mesh, ne*2 or ne*3
% bn : Boundary nodes in the refined mesh, nb*1
%--------------------------------------------------------------------
% (c) 2009, Koko J., ISIMA, koko@isima.fr
%--------------------------------------------------------------------
%
np=size(p0,1); nt=size(t0,1);

deps=sqrt(eps);
% Extract all edges
eh0=[t0(:,[1 2]); t0(:,[2 3]); t0(:,[1 3])];
eh=sort(eh0,2); [e,ie,je]=unique(eh,'rows');
% Form new nodes (mid-point of edges)
ne=size(e,1); p=[p0; (p0(e(:,1),:)+p0(e(:,2),:))/2];
% Form new edges
ip1=t0(:,1); ip2=t0(:,2); ip3=t0(:,3);
lnn=[(np+1):(np+ne)]'; lnnh=lnn(je);
mp1=lnnh(1:nt); mp2=lnnh(nt+1:2*nt); mp3=lnnh(2*nt+1:3*nt);
% Form new triangles
if (dg==1)
    t=zeros(4*nt,3);
    t(1:nt,:)=[ip1 mp1 mp3];
    t((nt+1):2*nt,:)=[mp1 ip2 mp2];
    t((2*nt+1):3*nt,:)=[mp2 ip3 mp3];
    t((3*nt+1):4*nt,:)=[mp1 mp2 mp3];
    % Force new boundary nodes
    neh=size(eh,1); he=full(sparse(je,1,ones(neh,1),ne,1)); ib=find(he==1);
    pb=p(np+ib,:); db=feval(fd,pb);
    ddx=(feval(fd,[pb(:,1)+deps,pb(:,2)])-db)/deps;
    ddy=(feval(fd,[pb(:,1),pb(:,2)+deps])-db)/deps;
    pb=pb-[db.*ddx,db.*ddy];
    p(np+ib,:)=pb;
elseif (dg==2)
    t=zeros(nt,6); t(:,1:3)=t0;
    t(:,4:6)=[mp1 mp2 mp3];
end
% Extract boundary edges & nodes
eh0=[t(:,[1 2]); t(:,[2 3]); t(:,[1 3])];
[eh,ih]=sort(eh0,2); [e,ie,je]=unique(eh,'rows');
ne=size(e,1); neh=size(eh,1);
he=full(sparse(je,1,ones(neh,1),ne,1)); ib=find(he==1);
bn=union(e(ib,1), e(ib,2));
be=e(ib,:);
if (dg==2)
    eq=[t(:,4); t(:,5); t(:,6)]; eq=eq(ih); eq=eq(ie);
    be=[e(ib,1), e(ib,2), eq(ib)];
    bn=union(bn,eq(ib));
end