%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% this program solves Darcy equation on a square using quadratic finite element %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Abdellahi SOUEID AHMED, %
% Abderrahim JARDANI,     %
% Andr REVIL,            %
% 09/24/2012              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
clc

%% globlal variables

global p
global t
global gg
global rho

%% constants
rho=1000; %water density
gg=9.82; %gravity
visco=1e-3; %water viscosity
%% goemetry

fd=inline('drectangle(p,0,100,0,100)','p');
fGam=inline('-1','p');
f=inline('0','p');
h0=1.8;
%
xmin=0;xmax=100;ymin=0;ymax=100;
[p,t]=kmg2d(fd,@huniform,h0,[xmin,ymin;xmax,ymax],2,1,[xmin,ymin;xmin,ymax;xmax,ymin;xmax,ymax]);
%% boundaries

epsilon=.001*h0;

top=( abs(p(:,2)-ymax)<epsilon );
bottom=( abs(p(:,2)-ymin)<epsilon );
left=( abs(p(:,1)-xmin)<epsilon );
right=( abs(p(:,1)-xmax)<epsilon );


gammaNV=left|right;  %Neumann border
gammaDV=top|bottom;%Dirichlet border

%values of Neumann and Dirichlet boundary conditions
zN=zeros(size(p,1),1);

zD=zeros(size(p,1),1);
zD(top)=3;
zD(bottom)=1;


%sink source
Q=zeros(size(p,1),1);
%% Darcy's equation solution
S=ones(size(t,1),1);
[A,b,Pressure]=Quasi_staticP2(Q,gammaDV,p,t,S,zD,zN,1);

%% solution's plot

plot_solution(t,p,Pressure,'P2');


