%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% this program solves Darcy's equation  coupled with potential equation for %
% the dam study case                                                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Abdellahi SOUEID AHMED, %
% Abderrahim JARDANI,     %
% Andr REVIL,            %
% 09/24/2012              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
clc
%% global variables

global p
global t
global gg
global rho
global visco

%% constants
gg=9.82;
rho=1000;
visco=1e-3;
r1=1; % anisotropy coef
r2=.01;

%% geometry
h0=.1;
load('dam_parameters','parameters');
p=parameters.p;
t=parameters.t;

g=center(p,t);

epsilon=.001*h0;

[ e, te ] = edgelist( p, t);
test = findinterior(p, e, te );
gamma=~test;

x=p(:,1); y=p(:,2);

left= gamma &( x>=69.1451702283296 & x<=69.7557599473453 ) & (y>=-6.26409495548962);
right= gamma &( x>=104.29125& x<=129.8125  & y>=-8.65459940652819);

%Dirichlet boundary conditions
gammaD=gamma &(left|right);

%values of Neumann and Dirichlet boundary conditions for hydraulic problem
zD=zeros(size(p,1),1);
zD(left)=(4+0.17*107)*9820;
zD(right)=4*9820;

zN=zeros(size(p,1),1);
%% hydraulic problem resolution

%hydraulic conductivity
KK=parameters.K;
% sink source
Q=zeros(size(p,1),1);

%Pressure
K=KK/(rho*gg);
[A,b,Pressure]=Quasi_static(Q,gammaD,p,t,K,zD,zN,0,r1,r2);
%% velocity
[vx,vy]=Velocity(p,t,Pressure,K',r1,r2, 'P1');

%% Compute Js_x and Js_y (current density)
Qv=parameters.Qv;
Jsx=Qv.*vx; Jsy=Qv.*vy;


JS_x=griddata(g(:,1),g(:,2),Jsx(:),x,y,'nearest');
JS_y=griddata(g(:,1),g(:,2),Jsy(:),x,y,'nearest');

%divergence of current density
[divJS]=Divergence_Operator(p,t,JS_x(:),JS_y(:),'P1');

%% Electrical problem boundaries

bottomV=( abs(y-(-20.6890207715134))<epsilon );
leftV=( abs(x-(-0.0487500000000082))<epsilon );
rightV=( abs(x-(129.8125))<epsilon );

gammaDV=   gamma & leftV|bottomV|rightV;
gammaN=gamma & (~gammaDV);

% electric boundary values
zD=zeros(size(p,1),1);
zD(gammaDV)=0;

zN=zeros(size(p,1),1);
zN(gammaN)=0;

%anisotropy coef
r1=1 ;
r2=.1/2;
%% Electric potential equation solution plot

%electrical conductivity
sigma=parameters.sigma;

%computing streaming potential
[A,b,V_cal]=Quasi_static(-divJS,gammaDV,p,t,sigma,zD,zN,0,r1,r2);

%% Streaming potential plot

clf()
plot_solution(t,p,V_cal,'P1');