%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% this program solves inverse problem for a  synthetic test case %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Abdellahi SOUEID AHMED, %
% Abderrahim JARDANI,     %
% Andr REVIL,            %
% 09/24/2012              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
clc

%% global variables
global p
global t
global zNV
global zDV
global gammaDV
global Cc
global xmin
global xmax
global ymin
global ymax
global XX
global YY
global nx
global nz
global A
global dx
global dz
global x
global z
global zobs
global top
global measures
global A_t
global b_t
global x_initial


%% goemetry

fd=inline('drectangle(p,0,100,0,50)','p');
fGam=inline('-1','p');
f=inline('0','p');
h0=1;
%
xmin=0;xmax=100;ymin=0;ymax=50;
[p,t]=distmesh2d(fd,@huniform,h0,[xmin,ymin;xmax,ymax],[xmin,ymin;xmin,ymax;xmax,ymin;xmax,ymax]);


%% boundaries

epsilon=.001*h0;

top=( abs(p(:,2)-ymax)<epsilon );
bottom=( abs(p(:,2)-ymin)<epsilon );
left=( abs(p(:,1)-xmin)<epsilon );
right=( abs(p(:,1)-xmax)<epsilon );


gammaNV=top; %Neumann border
gammaDV=left|right|bottom; %Dirichlet border

%values of Neumann and Dirichlet boundary conditions
zDV=zeros(size(p,1),1);
zNV=zeros(size(p,1),1);

%% inverse problem rectangular discretization

dx =11;
dz=5.5;

zobs=50; %observation level

x = (xmin+dx/2):dx:(xmax-dx/2);
z = (ymin+dz/2):dz:(ymax-dz/2);

nx=length(x);
nz=length(z);
[xm, zm] = meshgrid(x,z);
m=size(xm,1);
n=size(zm,2);
XX=reshape(xm,m*n,1);
YY=reshape(zm,m*n,1);

%% Kernel's computing

% measure points
%measures points are choosen on Neumann border
ind=find(top==1);
measures=ind(7:1:size(ind,1));

% exact synthetic external current density source
exact_source=zeros(2*m*n,1);
exact_source(53+9)=1;
exact_source(43+9)=1;
exact_source(33+9)=1;
exact_source(52-3*9)=1;
exact_source(51-3*9)=1;
exact_source(50-3*9)=1;
exact_source(42+81)=1;

%electrical conductivity
Conductivity=load('heterogenous_sigma' ,'sigma');
Cc=Conductivity.sigma;


[Stiff_mat,var1,var2]=Quasi_static(zeros(size(p,1),1),gammaDV,p,t,Cc,zDV,zNV,0);
[A]=Fast_Kernel(m,n,Stiff_mat);
[Vmes,JnMes]=Forward_Problem(exact_source);


%%%%% kernel matrix test
test_kernel=rand(2*m*n,1);
ekernel=A*test_kernel;
SP=Forward_Problem(test_kernel);

%error should tend to zero
error=max(SP-ekernel);
%% inverse problem

b=Vmes+1e-4*rand(size(Vmes,1),1);

L=Regularization_term('Z-smoothing-weighted',1.5);

%transformation into standard form
[A_t,b_t,K,M,L_p]=Transformation(A,b,L);
[U,S,V] = csvd(A_t);
x_initial=zeros(size(V,1),1);
[transformed_solution]=Regularization('TIKHONOV','GCV');

%inverse problem solution
[solution]=Transformation_back(L_p,transformed_solution,A,b,K,M);
%% plot inverse problem solution
clf()
inversion_plot(solution,exact_source);