%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% this program solves inverse problem for the geothermal study case %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Abdellahi SOUEID AHMED, %
% Abderrahim JARDANI,     %
% Andr REVIL,            %
% 09/24/2012              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
clc
%% global variables
global p
global t
global zN
global zD
global gammaD
global gammaDV
global Cc
global xmin
global xmax
global ymin
global ymax
global XX
global YY
global n
global m
global A
global dx
global dz
global x
global z
global zobs
global top
global measures
global A_t
global b_t
global x_initial
global zDV
global zNV

%% constants
gg=9.82; %gravity
visco=1e-3; %water viscosity
r1=1;r2=1; % anisotropy coefficients

%% goemetry
ymin=0;
ymax=354;
xmin=0;
xmax=1300;
h0=8;
pv=[0 0;1300 0;1300 354; 0 150; 0 0];
%[p,t]=distmesh2d(@dpoly,@huniform,h0,[0,0; 1300,354],pv,pv);

parameters=load('geothermal_parameters','param');
p=parameters.param.p;
t=parameters.param.t;

g=center(p,t);%elements centers

%% boundaries


epsilon=.001*h0;

% boundary nodes
[ e, te ] = edgelist( p, t);
test = findinterior(p, e, te );
gamma=~test;

bottom=( abs(p(:,2)-ymin)<epsilon );
left=( abs(p(:,1)-xmin)<epsilon );
right=( abs(p(:,1)-xmax)<epsilon );
top=gamma & p(:,2)>=150;
gamma0=bottom & (p(:,1)>=544 & p(:,1) <=710);
gamma1=left & (p(:,2)>=120 & p(:,2) <=150);
gammaD=gamma0|gamma1;%Dirichlet border, all remaining boundariesare Neumann ones



%values of Neumann and Dirichlet boundary conditions
zD=zeros(size(p,1),1);
zD(gamma0)=1e4;
zD(gamma1)=-1e2;
zN=zeros(size(p,1),1);

gammaDV=bottom|left|right;%electric Dirichlet border

zDV=zeros(size(p,1),1);
zNV=zDV;

%% inverse problem rectangular discretization

dx=80;
dz=22;
%dx=350;
%dz=150;

x = (xmin+dx/2):dx:(xmax-dx/2);
z = (ymin+dz/2):dz:(ymax-dz/2);
[xm, zm] = meshgrid(x,z);
m=size(xm,1);
n=size(zm,2);
XX=reshape(xm,m*n,1);
YY=reshape(zm,m*n,1);

%% a priori model

%electrical conductivity
Cc=parameters.param.conductivity;

%permability
permeability=parameters.param.permeability;


% sink source
Q=zeros(size(p,1),1);

%pressure distribution
[AA,b,p_cal]=Quasi_static(Q,gammaD,p,t,permeability,zD,zN,0);
%plot_solution(t,p,p_cal,'P1')

%Darcy's velocity
[vx,vy]=Velocity(p,t,p_cal,permeability,r1,r2, 'P1');

%density of charges
Log_QV=-9.2-0.821*log10(permeability);
Qv=10.^Log_QV;

%external current source
Jsx=Qv.*vx; Jsy=Qv.*vy;
JS_x=griddata(g(:,1),g(:,2),Jsx(:),p(:,1),p(:,2),'nearest');
JS_y=griddata(g(:,1),g(:,2),Jsy(:),p(:,1),p(:,2),'nearest');

%% Kernel's computing

% measure points
Vmes=xlsread('SP_geo.xls');
pts_measures=[];
for i=1:size(Vmes,1);
    pts_measures=[pts_measures; [nearest_point_border(Vmes(i,1),top,p,110) , Vmes(i,2)]];
end

[m1,u,v] = unique(pts_measures(:,1),'rows');
m2 = accumarray(v,pts_measures(:,2),[],@mean);
measures = [m1 m2];

%kernel
[A]=Kernel(m,n);

%%%%% kernel matrix test
test_kernel=rand(2*m*n,1);
ekernel=A*test_kernel;
SP=Forward_Problem(test_kernel);

%error should tend to zero
error=max(SP-ekernel);
%% inverse problem

b2=measures(2:end,2);%+2*rand(59,1);

mx=griddata(g(:,1),g(:,2),Jsx,XX,YY,'nearest');
my=griddata(g(:,1),g(:,2),Jsy,XX,YY,'nearest');
model_a_priori=[mx;my];
x_initial=model_a_priori;

zobs=mean(measures(:,1));%observation level
L=Regularization_term('weighted1',3);

[A_t,b_t,K,M,L_p]=Transformation(A,b2,L);
[U,S,V] = csvd(A_t);
[transformed_solution1]=Regularization('TIKHONOV','GCV');
%[lambda] =l_curve(U,S,b_t);

transformed_solution =Transformation_back(L_p,transformed_solution1,A,b2,K,M);

%inverted current density
solution = Transformation_back(L_p,transformed_solution,x_initial);

%norm of inverted current density
norm_solution=sqrt(solution(1:n*m).^2+solution(n*m+1:end).^2);

% norm of a priori current density model
norm_a_priori=sqrt(mx.^2+my.^2);

%norm of Darcy's velocity
%%
% velocity's inversion

%density of charges over rectangular inverse prolem discretization
Qv_blocks=griddata(g(:,1),g(:,2),Qv,XX,YY,'nearest');
Qv_blocks(123)= Qv_blocks(123)/2;
Qv_blocks(122)= Qv_blocks(122)/1.5;
Qv_blocks(121)= Qv_blocks(121)/1.5;
Qv_blocks(122+16)= Qv_blocks(122+16)/7;
Qv_blocks(121+16)= Qv_blocks(121+16)/3;

ux=solution(1:n*m)./Qv_blocks;
uy=solution(n*m+1:end)./Qv_blocks;

%norm of velocity
norm_velocity=sqrt(ux.^2+uy.^2);

%%
%% plot of inverted velocity solution ( same fucntion can be used to
%  plot inverted current density
clf()
plot_vector(norm_velocity);