%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% this program solves Darcy's equation  coupled with potential equation for %
% the volcano study case                                                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Abdellahi SOUEID AHMED, %
% Abderrahim JARDANI,     %
% Andr REVIL,            %
% 09/24/2012              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
clc
%% global variables


global p
global t
global gg
global rho
global visco

%% constants
gg=9.82;
rho=1000;
visco=1e-3;
r1=1; % anisotropy coef
r2=1;

%% geometry
h0=1.5;
ymin=0;

parameters=load('volcano_parameters','param');
p=parameters.param.p;
t=parameters.param.t;

Log10_hydraulic_cond=parameters.param.K;
sigma=parameters.param.sigma;
%% boundaries

epsilon=.001*h0;

[ e, te ] = edgelist( p, t);
test = findinterior(p, e, te );
gamma=~test;

x=p(:,1); y=p(:,2);
xmin=0; xmax=1e4;

bottom=( abs(y-ymin)<epsilon );
gamma0=bottom & (x>4950&  x<5430);
gamma1= gamma & (x>3350  &  x<3430) & (y>1706 &  y<1849);
gamma2 = gamma &(x>5000 &  x<5430) & (y>2500 &  y<3000);

borders=gamma &(gamma0|gamma1|gamma2);
reste=gamma & (~borders);
gammaD=borders; %Dirichlet's boundaries

%values of Neumann and Dirichlet boundary conditions
zD=zeros(size(p,1),1);
zD(gamma0)=-1e4;
zD(gamma1)=1e4;
zD(gamma2)=1e4;

zN=zeros(size(p,1),1);
zN(reste)=0;

%% hydraulic problem resolution

% sink source
Q=zeros(size(p,1),1);

%Pressure
[A,b,Pressure]=Quasi_static(Q,gammaD,p,t,Log10_hydraulic_cond,zD,zN,1);

%% velocity

K=10.^(-Log10_hydraulic_cond)';
K=K/(rho*gg);

[vx,vy]=Velocity(p,t,Pressure,K,r1,r2, 'P1');

%% Compute Js_x and Js_y (current density)
K_permeability=(visco*K);

%density of charges
Log_QV=-9.2-0.821*log10(K_permeability);
Qv=10.^Log_QV;

Jsx=Qv.*vx; Jsy=Qv.*vy;
g=center(p,t);


JS_x=griddata(g(:,1),g(:,2),Jsx(:),x,y,'nearest');
JS_y=griddata(g(:,1),g(:,2),Jsy(:),x,y,'nearest');

[divJ]=Divergence_Operator(p,t,JS_x(:),JS_y(:),'P1');
divJS=griddata(x,y,divJ(:),g(:,1),g(:,2),'nearest');

%% electric  boundary conditions

bottomV=( abs(y-ymin)<epsilon );
gamma0V=bottom & (x>4950&  y<5430);
gamma1V= gamma & (x>3350  &  x<3430) & (y>1706 &  y<1849);
gamma2V = gamma &(x>4900 &  x<5459) & (y>2500 &  y<3000);
gamma2Vg=      gamma & (x>5000 &  x<5176) & (y>2500 &  y<3000);
gamma2Vd= gamma & (x>5176 &  x<5430) & (y>2500 &  y<3000);
rightV=( abs(x-xmax)<epsilon );
leftV=( abs(x-xmin)<2*h0 );
right_bottomV=bottomV & (x>5459 & x<xmax );
left_bottomV=bottomV & (x>0 & x<4900 );
gammaDV=   gamma & rightV|leftV|right_bottomV|left_bottomV;
resteV=gamma & (~gammaDV) & (~(gamma0V|gamma1V|gamma2Vg|gamma2Vd));




%indices of Neumann's nodes
indiceg0=find_indice(p,gamma0V,2);
indiceg1=find_indice(p,gamma1V);
indiceg2g=find_indice(p,gamma2Vg);
indiceg2d=find_indice(p,gamma2Vd);

%normal orientation's over Neumann's border
[normalsg0]=normal_orientation_border(p,t,indiceg0);
normalsg0=[normalsg0(:,1) normalsg0];
[normalsg1]=normal_orientation_border(p,t,indiceg1);
normalsg1=[normalsg1(:,1) normalsg1];
[normalsg2g]=normal_orientation_border(p,t,indiceg2g);
normalsg2g=[normalsg2g(:,1) normalsg2g];
[normalsg2d]=normal_orientation_border(p,t,indiceg2d);
normalsg2d=[normalsg2d(:,1) normalsg2d];


% electric boundary values
zD=zeros(size(p,1),1);
zD(gammaDV)=0;


%Neumann booundary conditions : the velue 0.0027 represents the electrical
%conductivity on Neumann boundaries.
zN=zeros(size(p,1),1);
zN(gamma0V)=normalsg0(1,:)'.*JS_x(gamma0V)./0.027+normalsg0(2,:)'.*JS_y(gamma0V)./0.027;
zN(gamma1V)=normalsg1(1,:)'.*JS_x(gamma1V)./0.027+normalsg1(2,:)'.*JS_y(gamma1V)./0.027;
zN(gamma2Vg)=normalsg2g(1,:)'.*JS_x(gamma2Vg)./0.027+normalsg2g(2,:)'.*JS_y(gamma2Vg)./0.027;
zN(gamma2Vd)=normalsg2d(1,:)'.*JS_x(gamma2Vd)./0.027+normalsg2d(2,:)'.*JS_y(gamma2Vd)./0.027;

%% Electric potential equation solution plot
[A,b,V_cal]=Quasi_static(divJS,gammaDV,p,t,sigma,zD,zN,0);

%% Pressure and Streaming potential plots

clf()
subplot(2,1,1)
plot_solution(t,p,Pressure,'P1');
subplot(2,1,2)
plot_solution(t,p,V_cal,'P1');