%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%computes discrete divergence                          %
%inputs                                                %
%       p : nodes coordinates                          % 
%       t : nodes connectivities                       %
%       Ux : variable x-component                      %
%       Uy : variable y-component                      %
%       method : finite element method (P1 or P2)      %
%outputs                                               %
%       divU : divergence of the variable U=[Ux,Uy];   % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Abdellahi SOUEID AHMED, %
% Abderrahim JARDANI,     %
% Andr REVIL,            %
% 09/24/2012              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [divU]=Divergence_Operator(p,t,Ux,Uy,method)

n=size(t,1);
gUx=ones(n,1);
gUy=gUx;

switch method
    case 'P1'
        for iel =1:n
            j=t(iel,1); k=t(iel,2); l=t(iel,3);
            J=[p(k,1)-p(j,1), p(l,1)-p(j,1); p(k,2)-p(j,2), p(l,2)-p(j,2)];
            jacobian=inv(J');
            gUx(iel,1)=jacobian(1,:)*([-1; -1]*Ux(j)+[1; 0]*Ux(k)+[0 ;1]*Ux(l)) ;
            gUy(iel,1)=jacobian(2,:)*([-1; -1]*Uy(j)+[1; 0]*Uy(k)+[0 ;1]*Uy(l)) ;
        end
        divU=gUx+gUy;
        g = center(p,t);
        divU = griddata(g(:,1),g(:,2),divU,p(:,1),p(:,2),'nearest');
        
    case 'P2'
        for iel =1:n            
            %(j,k,l) : element in question
            j=t(iel,1); k=t(iel,2); l=t(iel,3); jk=t(iel,4); kl=t(iel,5); jl=t(iel,6);
            J=[p(k,1)-p(j,1), p(l,1)-p(j,1); p(k,2)-p(j,2), p(l,2)-p(j,2)];
            jacobian=inv(J');
            gUx(iel,1)=jacobian(1,:)*(Ux(j)*[-3;-3]+Ux(k)*[-1;0] ...
                +Ux(l)*[0;-1]+Ux(jk)*[4;0]+Ux(kl)*[0;0]+Ux(jl)*[0;4]);
            gUy(iel,1)=jacobian(2,:)*(Uy(j)*[-3;-3]+Uy(k)*[-1;0] ...
                +Uy(l)*[0;-1]+Uy(jk)*[4;0]+Uy(kl)*[0;0]+Uy(jl)*[0;4]);   
        end
        divU=gUx+gUy;
    otherwise
        error('non implemented finite element method')
end
