%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%computes forward problem solution for the inverse problem         %
%input :                                                           %
%        data : external current density Js                        %
%                                                                  %  
%outputs :                                                         %
%          SP : Streaming potential computed in measurement points     %
%          V :  Streaming potential over all mesh points           %
%          V-ref : Streaming potential of reference electrode      %
%          J : divergence of external current density              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Abdellahi SOUEID AHMED, %
% Abderrahim JARDANI,     %
% Andr REVIL,            %
% 09/24/2012              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [SP,V,V_ref,J]=Forward_Problem(data)

global p
global t
global gammaDV 
global Cc
global zDV
global zNV
global measures

[Jxx,Jyy]=interpolation(data); 
[J]=Divergence_Operator(p,t,Jxx,Jyy,'P1');
[A,b,V]=Quasi_static(J,gammaDV,p,t,Cc,zDV,zNV,0);

points=measures(:,1);
n=size(points,1);
V_ref=V(points(1));
SP= V(points(2:end))-ones(n-1,1)*V_ref;
J=J(points);

