%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%computes discrete gradient                            %
%inputs                                                %
%       p : nodes coordinates                          % 
%       t : nodes connectivities                       %
%       u : variable                                   %
%       method : finite element method (P1 or P2)      %
%outputs                                               %
%       gradUx : x-gradient                            % 
%       gradUy : y-gradient                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Abdellahi SOUEID AHMED, %
% Abderrahim JARDANI,     %
% Andr REVIL,            %
% 09/24/2012              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [gradUx,gradUy]=Gradient_operator(p,t,u, method)

n=size(t,1);
gradUx=zeros(n,1);
gradUy=gradUx;

switch method
    case 'P1'        
        for iel =1:n
            %(j,k,l) : element in question
            j=t(iel,1); k=t(iel,2); l=t(iel,3);
            J=[p(k,1)-p(j,1), p(l,1)-p(j,1); p(k,2)-p(j,2), p(l,2)-p(j,2)];
            jacobian=inv(J');
            gradUx(iel,1)=jacobian(1,:)*([-1; -1]*u(j)+[1; 0]*u(k)+[0 ;1]*u(l)) ;
            gradUy(iel,1)=jacobian(2,:)*([-1; -1]*u(j)+[1; 0]*u(k)+[0 ;1]*u(l)) ;
        end
        
    case 'P2'
        for iel =1:n
            %(j,k,l) : element in question
            j=t(iel,1); k=t(iel,2); l=t(iel,3); jk=t(iel,4); kl=t(iel,5); jl=t(iel,6);
            J=[p(k,1)-p(j,1), p(l,1)-p(j,1); p(k,2)-p(j,2), p(l,2)-p(j,2)];
            jacobian=inv(J');
            gradUx(iel,1)=jacobian(1,:)*(u(j)*[-3;-3]+u(k)*[-1;0] ...
                +u(l)*[0;-1]+u(jk)*[4;0]+u(kl)*[0;0]+u(jl)*[0;4]);
            gradUy(iel,1)=jacobian(2,:)*(u(j)*[-3;-3]+u(k)*[-1;0] ...
                +u(l)*[0;-1]+u(jk)*[4;0]+u(kl)*[0;0]+u(jl)*[0;4]);
        end
        
    otherwise
        error('non implemented finite element method')
end