%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%computes  right hand-side and solution U   for the    %
% equation -div(S*grad(U))=Q  while stiffness matrix is already known   %
%and given as input                                                     %
%inputs :                                                               %
%         K_global : stiffness matrix                                   %
%         Q : sink  source                                              %
%         GammaD : Dirichlet border                                   %
%         coord : nodes coordinates                                     %
%         nodes : connectivites                                         %
%         S :  hydraulic conductivity  or its decimal logarithm         %
%                                                                       %
%         zD : values of P on Dirichlet border                        %
%         zN : values of Neumann border                               %
%         boolean : 1 if we use logarithm of hydraulic conductivity     %
%                   0 otherwise                                         %
%outputs :                                                              %
%         K_global : matrix problem                                     %
%         second_term: right-hand side                                  %
%         U : solution                                                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Abdellahi SOUEID AHMED, %
% Abderrahim JARDANI,     %
% Andr REVIL,            %
% 09/24/2012              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [U]=Quasi_static_bis(K_global,Q,GammaD,coord,nodes,S,zD,zN,boolean,r1,r2)

global gg  %gravity constant
global rho % volumic mass

[ e, te ] = edgelist( coord, nodes);
test = findinterior( coord, e, te ); %interior nodes

GammaD1=(GammaD) &(~test) ; %Dirichlet's points
GammaN1=(~test)&(~GammaD1); % Neumann points

nbN=size(coord,1); %nombre total de noeuds
U=zeros(nbN,1);
f=U;
UNmann=U; %nbN nombre total de noeuds

N_unk=size(find(~GammaD1==1),1); %Number of unknowns
UNmann(~GammaD1)=(1:1:N_unk)';

%evaluates U on Dirichlet's borders
qq=scdm(coord,Q);
for j=1:nbN
    f(j)=qq(j,1);
    if GammaD1(j)
        U(j)=zD(j);
    end
end

%initialization
 second_term=zeros(N_unk,1);

centers=center(coord,nodes);

 if boolean==1
     kappa=10.^(-S);
  kappa=griddata(centers(:,1),centers(:,2),kappa(:),coord(:,1),coord(:,2),'nearest');
 else
     kappa=S;
     kappa=griddata(centers(:,1),centers(:,2),kappa(:),coord(:,1),coord(:,2),'nearest');
 end

%loop over elements
for iel=1:size(nodes,1)
    %(j,k,l) : element in question
    j=nodes(iel,1); k=nodes(iel,2); l=nodes(iel,3);

    %UNmann(a)>0 if a is unknown
    jj=UNmann(j); kk=UNmann(k); ll=UNmann(l);

    belem=[f(j) f(k) f(l)];

    % coordinates of each vertex
    x11=coord(j,1);
    x12=coord(j,2);
    x21=coord(k,1);
    x22=coord(k,2);
    x31=coord(l,1);
    x32=coord(l,2);

    %jacobian(determinant of jacobian matrix)
    jac=abs((x11-x31).*(x32-x22)-(x12-x32).*(x31-x21));

    %gadients of basis functions phii,phij,phik

    %gadient of phii
    gphiix=-(x32-x22)./jac;
    gphiiy=(x31-x21)./jac;
    dphi=[gphiix ; gphiiy];

    %gadient of phij
    gphijx=-(x12-x32)./jac;
    gphijy=(x11-x31)./jac;
    dphij=[gphijx ; gphijy];

    %gadient of phik
    gphikx=-(x22-x12)./jac;
    gphiky=(x21-x11)./jac;
    dphik=[gphikx ; gphiky];

    if nargin == 9

        %elementary stiffness matrix
        k11=dot(dphi,dphi);
        k12=dot(dphi,dphij);
        k13=dot(dphi,dphik);
        k22=dot(dphij,dphij);
        k23=dot(dphij,dphik);
        k33=dot(dphik,dphik);

        %symetry
        k21=k12;
        k31=k13;
        k32=k23;

        if boolean==1
            Kel=0.5*jac*(10^(-S(iel))/(gg*rho))*[k11 k12 k13 ; k21 k22 k23 ; k31 k32 k33];
        else
            Kel=0.5*jac*S(iel)*[k11 k12 k13 ; k21 k22 k23 ; k31 k32 k33];
        end
    else

        k11=r1*gphiix*gphiix+r2*gphiiy*gphiiy;
        k12=r1*gphiix* gphijx+r2*gphiiy*gphijy;
        k13=r1*gphiix* gphikx+r2*gphiiy*gphiky;
        k22=r1*gphijx* gphijx+r2*gphijy*gphijy;
        k23=r1*gphijx* gphikx+r2*gphijy*gphiky;
        k33=r1*gphikx* gphikx+r2*gphiky*gphiky;

        %symetry
        k21=k12;
        k31=k13;
        k32=k23;

        if boolean==1
            Kel=0.5*jac*(10^(-S(iel))/(gg*rho))*[k11 k12 k13 ; k21 k22 k23 ; k31 k32 k33];
        else
            Kel=0.5*jac*S(iel)*[k11 k12 k13 ; k21 k22 k23 ; k31 k32 k33];
        end
    end


    cste=jac/24;

    %Computing right-hande side 

    % f is projected on (phii, phij,phik) basis
    if jj>0
        second_term(jj)=second_term(jj)+cste*belem*[2;1;1];
    else
        if kk>0
            %part of b involving stiffness matrix terms
            second_term(kk)=second_term(kk)-U(j)*Kel(1,2);
        end
        if ll>0
            second_term(ll)=second_term(ll)-U(j)*Kel(1,3);
        end
    end

    if kk>0
        second_term(kk)=second_term(kk)+cste*belem*[1;2;1];
    else
        if jj>0,
            second_term(jj)=second_term(jj)-U(k)*Kel(1,2);
        end
        if ll>0,
            second_term(ll)=second_term(ll)-U(k)*Kel(2,3);
        end
    end

    if ll>0
        second_term(ll)=second_term(ll)+cste*belem*[1;1;2];
    else
        if jj>0,
            second_term(jj)=second_term(jj)-U(l)*Kel(1,3);
        end
        if kk>0,
            second_term(kk)=second_term(kk)-U(l)*Kel(2,3);
        end
    end

    %in case we have Neumann contributions, we have to add its contributions
    if GammaN1(j)% in Neumann's border
        if test(k)==0, %on boundaries
            %1D shape functions evaluation on  GammmaN
           second_term(jj)=second_term(jj)+Neumann_border(coord(j,:),coord(k,:),zN(j));
        end
        if test(l)==0,
           second_term(jj)=second_term(jj)+Neumann_border(coord(j,:),coord(l,:),zN(j));
        end
    end

    if GammaN1(k)
        if test(j)==0,
           second_term(kk)=second_term(kk)+Neumann_border(coord(k,:),coord(j,:),zN(k));
        end
        if test(l)==0,
           second_term(kk)=second_term(kk)+Neumann_border(coord(k,:),coord(l,:),zN(k));
        end
    end

    if GammaN1(l)
        if test(j)==0,
           second_term(ll)=second_term(ll)+Neumann_border(coord(l,:),coord(j,:),zN(l));
        end
        if test(k)==0,
           second_term(ll)=second_term(ll)+Neumann_border(coord(l,:),coord(k,:),zN(l));
        end
    end
    
end

U(~GammaD1)=K_global\second_term; %solution of the problem

