%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%this function interpolates electric current density (defined on a grid)  %
%on a finite element mesh                                                 %
%input                                                                    %
%       data : electric current density                                  %
%outputs                                                                  %
%       Jxx , Jyy :  x and y components of interpolated electric current     %
%       density                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Abdellahi SOUEID AHMED, %
% Abderrahim JARDANI,     %
% Andr REVIL,            %
% 09/24/2012              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%



function [Jxx,Jyy]=interpolation(data)


global p
global xmin
global xmax
global ymin
global ymax
global dx
global dz


x = (xmin+dx/2):dx:(xmax-dx/2);
z = (ymin+dz/2):dz:(ymax-dz/2);


[xm, zm] = meshgrid(x,z);
m=size(xm,1);
n=size(zm,2);
XX=reshape(xm,m*n,1);
YY=reshape(zm,m*n,1);

data_x=data(1:m*n);

data_y=data(m*n+1:2*m*n);
Fx = TriScatteredInterp(XX,YY,data_x,'nearest');
Jxx=Fx(p(:,1),p(:,2));

Fy = TriScatteredInterp(XX,YY,data_y,'nearest');
Jyy=Fy(p(:,1),p(:,2));