
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%computes outward normal (using cosine directions) on an edge   %
%                                                               %
%inputs                                                         %          
%        p : mesh nodes                                        %
%        t : mesh connectivities                                %  
%        edge : edge on which we look for the outward normal    %
%        iel : index of element to which belongs the edge        %    
%output                                                         %
%       n : cosine direction                                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Abdellahi SOUEID AHMED, %
% Abderrahim JARDANI,     %
% Andr REVIL,            %
% 09/24/2012              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%



function [n] = normal_orientation(t,p,edge,iel)
%(j,k,l) : element in question

j=t(iel,1); k=t(iel,2); l=t(iel,3);
 %jacobian
J=[p(k,1)-p(j,1), p(l,1)-p(j,1); p(k,2)-p(j,2), p(l,2)-p(j,2)];
J=inv(J);

%cosine direction of reference triangle
X=[0  .5 -1 ; -1 sqrt(3)/2 0];

if (edge(1)==j & edge(2)==k) | (edge(1)==k & edge(2)==j)
     
    n=J'*X(:,1);
    Norm=sqrt(n(1)^2+n(2)^2);
    n=n/Norm;
elseif (edge(1)==k & edge(2)==l) | (edge(1)==l & edge(2)==k)

    n=J'*X(:,2);
    Norm=sqrt(n(1)^2+n(2)^2);
    n=n/Norm;
elseif (edge(1)==j & edge(2)==l) | (edge(1)==l & edge(2)==j)

    n=J'*X(:,3);
    Norm=sqrt(n(1)^2+n(2)^2);
    n=n/Norm;
end



