%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function computes the regularized solution of                      %
% optmization problem                                                     %
%                                                                         %
%inputs                                                                   %
%       - method : regularization method, it can be Tikhonov method or    %
%        tsvd method (truncated singular value decomposition)             %
% - param_method : this parameter is computed by L_curve method or GCV     %
%                  method. When the used method is Tikhonov, param_method  %
%                  represents regularization parameter Lambda, otherwise  %
%                  it represents the TSVD truncation parameter.           %
%                                                                         %
%output                                                                   %
%       x : regularized solution of the inverse problem                   %
%                                                                         %
%                                                                         %
%                                                                         %
%References : Regularization Tools : A Matlab Package for                 %
%Analysis and Solution of Discrete Ill-Posed Problems                     %
%Per Christian Hansen                                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Abdellahi SOUEID AHMED, %
% Abderrahim JARDANI,     %
% Andr REVIL,            %
% 09/24/2012              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%
function [x]=Regularization(method,param_method)

global A_t
global b_t
global x_initial

[U,S,V] = csvd(A_t);
s = size(S,1);

%%
switch method
    case 'TIKHONOV' 
     
      lambda=Regularization_parameter(param_method,'TIKHONOV');
      x = V(:,1:s)*((S(:,1).*(U(:,1:s)'*b_t) + lambda^2*V'*x_initial)./(S.^2 + lambda^2));
   %%
   %%
    case  'TSVD'
        TSVD_param=Regularization_parameter2(param_method,'TSVD');
        rapport =  U(:,1:s)'*b_t./S;
        if TSVD_param<0
            error(' negative TSVD parameter ')
        else
            x = V(:,1:TSVD_param)*rapport(1:TSVD_param);
        end
     %%  
     
    otherwise 
        error('Non-existing regularization method')
       
end
