%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%computes regularization parameter, for Tikhonov method it is Lambda and  %
%for TSVD it represents the truncation parameter k                        %
%                                                                         % 
% inputs :  param_method : L_curve or GCV                                 %
%           reg_method : Tikhonov or TSVD                                 %
% Output :                                                                % 
%           reg_prameter : lambda or k                                    %
%                                                                         %
%                                                                         %
%References : Regularization Tools : A Matlab Package for                 %
%Analysis and Solution of Discrete Ill-Posed Problems                     %
%Per Christian Hansen                                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Abdellahi SOUEID AHMED, %
% Abderrahim JARDANI,     %
% Andr REVIL,            %
% 09/24/2012              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%
function [reg_parameter]=Regularization_parameter(param_method,reg_method)

global A_t
global b_t

[U, S, V] = svd(A_t);
btilde=U'*b_t;
S=diag(S);
%%
switch param_method
    case 'L_CURVE'
        
        reg_parameter=l_curve (U,S,b_t,reg_method);
        
        %%
        %%
    case  'GCV'
        
        if strcmp(reg_method, 'TIKHONOV')
            
            n = 200;                     
            min_parameter = 16*eps;             
            
            [Ur,Uc] = size(U); s = size(S,1);
            b = U'*b_t; 
            b2 = norm(b_t)^2 - norm(b)^2;
           
            parameters = zeros(n,1); G = parameters;  
            parameters(n) = max([S(s),S(1)*min_parameter]);
            rapport = (S(1)/parameters(n))^(1/(n-1));
           
            for i=n-1:-1:1
                parameters(i) =rapport*parameters(i+1);
            end
            
            residual = 0;
            if (Ur > Ur & b2 > 0)
                residual = b2;
            end

            for i=1:n
                filter = (parameters(i)^2)./( S.^2 + parameters(i)^2);
                G(i) = (norm(filter.*b(1:s))^2 + residual)/(Ur-Uc + sum(filter))^2;
            end
            [~,minimum] = min(G); % Initial guess.
            reg_parameter = fminbnd('Tikhonov_gcv_function',  parameters(min(minimum+1,n)),parameters(max(minimum-1,1)), [], btilde, S);
            
        elseif strcmp(reg_method, 'TSVD')
            
            [S, index] = sort(abs(S));
            S = flipud(S);
            b = abs( flipud( btilde(index) ) ) / sqrt(length(btilde));
            n = length(S);
            eta = zeros(n-1,1);
            eta(n-1) = b(n)^2;
            for i=n-2:-1:1
              eta(i) = eta(i+1) + b(i+1)^2;
            end
            G = zeros(n-1,1);
            for i=1:n-1
              G(k) = eta(i)/(n-i)^2;
            end
            [~,reg_parameter] = min(G);

        else
            error('non supported param_method for GCV');
        end
        %%
        
    otherwise
        error('Non-existing regUrarization param_method')
        
end








