%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function computes the regularization term L                        %
% for several kinds of weights and smoothing matrices                      %
%                                                                         %
%inputs                                                                   %
%       term : kind of regularization                                     %
%       s : weigthing matrix exponent                                     %
%                                                                         %
%output                                                                   %
%       L : regularization matrix                                         %
%                                                                         %
%                                                                         %
%References : Depth resolution studies for 3D geomagnetic problems        %
%Informatics and Mathematical Modelling, Technical University of          %
%Denmark,DTU                                                              %
%Marie Lund Andersen                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Abdellahi SOUEID AHMED, %
% Abderrahim JARDANI,     %
% Andr REVIL,            %
% 09/24/2012              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%
function [L]=Regularization_term(term,s)

global dx
global dz
global x
global z
global zobs
global A

nx=length(x);
nz=length(z);

%%
dxx=sqrt(dx)*ones(length(x),1);
Ix=diag(dxx);
dzz=sqrt(dz)*ones(length(z),1);
for i=1:nz-1
    hz(i)=.5*(dzz(i).^2+dzz(i+1).^2);
end

W = diag((zobs-z).^s);
Wz=kron(eye(nx,nx),W);
%%
switch term
    case 'weighted1' %W(z)||W(z)M(x,z)||2
        
        L=kron(eye(2,2),Wz);
        
    case 'weighted2' %W(z)||W(z)M(x,z)||2
        
        L=diag(sqrt(sum(A.*A))'); %A is the kernel
        
    case 'Z-smoothing-weighted' % || d(W(z)M(x,z))dzz||2
        
        [L] = get_l(nz,1);
        rapZ=dzz(1:nz-1)./hz';
        for i =1:size(L,1)
            L1z(i,:)=L(i,:)*rapZ(i);
        end
        L1z=full(L1z);
        L1z=kron(Ix,L1z)*Wz;
        L=kron(eye(2,2),L1z);
        
    case 'Z2-smoothing-weighted' % || d2(W(z)M(x,z))dzz2||2
        
        for i=1:nz-2
            hzbar(i)=.5*(hz(i)+hz(i+1));
        end
        rapZ1=dzz(2:nz-1)./(hzbar'.*dzz(1:nz-2));
        rapZ3=dzz(2:nz-1)./(hzbar'.*dzz(2:nz-1));
        rapZ2=rapZ1+rapZ3;
        [L2] = get_l(nz,2);
        ind= L2==-2;
        L2(ind)=-1;
        L2=full(L2);
        rapZbis=zeros(nz-2,nz);
        n=nz;
        for i=1:n-2
            rapZbis(i,:)=[zeros(1,i-1) , [rapZ1(i) rapZ2(2) rapZ3(3) ] , zeros(1,n-(length(zeros(1,i-1))+3))];
        end
        for i =1:size(L2,1)
            L2z(i,:)=L2(i,:).*rapZbis(i,:);
        end
        L2z=full(L2z);
        L02z=kron(Ix,L2z)*Wz;
        L=kron(eye(2,2),L02z);
        
    case  'XZ-smoothing-weighted'
        
        d=1*ones(nx*nz,1);
        d_1=1*ones(nx*nz-1,1);
        d_2=1*ones(nx*nz-nx,1);
        d_3=[zeros(nx*nz-2*nx,1);1*ones(nx,1)];
        D_x_z=diag(d,0)+diag(-0.5*d_1,1)+diag(-0.5*d_2,nx)+diag(-0.5*d_3,-nx);
        D_x_z(nx*nz,nx*nz-1)=-0.5;
        L=kron(eye(2,2),Wz)*kron(eye(2,2),D_x_z);
        
    case  'Identity'
        
        L=eye(nx*nz,nx*nz);
        L=kron(eye(2,2),L);
        
    otherwise
        error('Non existing regularization term')
end

%%
