%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%transforms a general inverse problem into a standard one                  %
%                                                                         %
%                                                                         %
%References : Regularization Tools : A Matlab Package for                 %
%Analysis and Solution of Discrete Ill-Posed Problems                     %
%Per Christian Hansen                                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [A_s,b_s,K,M,L_p]=Transformation(A,b,L)

  [m,n] = size(A); [p,~] = size(L);


  % Special treatment of the case where L is square.
  if (p==n)
    L_p = inv(L); K = []; M = []; A_s = A/L; b_s = b;
    
  else
  
   
  % Compute a QR factorization of L'.
  [K,R] = qr(full(L'));

  R = R(1:p,:);

  % Compute a QR factorization of A*K(:,p+1:n)).
  [H,T] = qr(A*K(:,p+1:n)); T = T(1:n-p,:);

  % Compute the transformed quantities.
  L_p = K(:,1:p)/R';   
  K   = K(:,p+1:n);
  M   = T\(H(:,1:n-p)');
  A_s = H(:,n-p+1:m)'*A*L_p;
  b_s = H(:,n-p+1:m)'*b;
  end
  
  